"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useViewport = void 0;
var React = require("react");
var useEffect = React.useEffect, useState = React.useState;
var getViewportWidth = function () {
    var docClientWidth = document.documentElement
        ? document.documentElement.clientWidth
        : 0;
    var documentBodyWidth = document.body ? document.body.clientWidth : 0;
    return window.innerWidth || docClientWidth || documentBodyWidth;
};
var getViewportHeight = function () {
    var documentElement = document.documentElement;
    var documentHeight = documentElement.clientHeight;
    return window.innerHeight || documentHeight;
};
var getDocumentHeight = function () {
    var documentElement = document.documentElement;
    var documentHeight = documentElement.scrollHeight;
    return documentHeight;
};
var useViewport = function () {
    var _a = useState(getViewportWidth()), viewportWidth = _a[0], setViewportWidth = _a[1];
    var _b = useState(getViewportHeight()), viewportHeight = _b[0], setViewportHeight = _b[1];
    var _c = useState(getDocumentHeight()), documentHeight = _c[0], setDocumentHeight = _c[1];
    useEffect(function () {
        window.addEventListener('resize', getWindowDimensions);
        window.addEventListener('scroll', getScrollPosition);
        return function () {
            window.removeEventListener('resize', getWindowDimensions);
            window.removeEventListener('scroll', getScrollPosition);
        };
    }, []);
    var getWindowDimensions = function () {
        setViewportWidth(getViewportWidth());
        setViewportHeight(getViewportHeight());
        setDocumentHeight(getDocumentHeight());
    };
    var getScrollPosition = function () {
        setDocumentHeight(getDocumentHeight());
    };
    return {
        documentHeight: documentHeight,
        viewportHeight: viewportHeight,
        viewportWidth: viewportWidth,
    };
};
exports.useViewport = useViewport;
